@@ -2,10 +2,9 @@ from web3 import Web3, Account as Account
 from eth_typing.evm import Address
 from web3.types import Wei, TxParams
 from eth_utils.conversions import to_hex
-from web3.contract.contract import ContractFunction
-from web3.contract import Contract
+from web3.contract.async_contract import AsyncContractFunction
+from web3.contract import AsyncContract
 from typing import Optional, Tuple
-import requests
 import os
 from loguru import logger
 from pathlib import Path
@@ -15,12 +14,16 @@ from .proxy import get_dynamic_http_proxy, get_static_http_proxy
 from .const import STATUS
 from .network import get_rpc, get_chain_id
 import random
+from web3 import AsyncWeb3
+from web3.providers import AsyncHTTPProvider
+import asyncio
+from async_http import AsyncHTTPWithProxyProvider
 
 
 class EvmAccount:
     name: str
     private_key: str
-    w3: Web3
+    w3: AsyncWeb3
     address: Address
     account: Account
     gas_price_multiplier: float
@@ -47,7 +50,7 @@ class EvmAccount:
             raise ValueError("rpc config cant be empty")
         if not endpoint:
             rpc_list = get_rpc(name=endpoint_name)
-            endpoint = rpc_list[random.randint(0, len(rpc_list - 1))]
+            endpoint = rpc_list[random.randint(0, len(rpc_list) - 1)]
             self.chain_id = get_chain_id(name=endpoint_name)
         if chain_id is not None:
             self.chain_id = chain_id
@@ -107,26 +110,18 @@ class EvmAccount:
         self,
         use_proxy,
         is_static_proxy: bool,
-        proxy: dict,
+        proxy: str,
         endpoint: str = None,
     ):
-        adapter = requests.adapters.HTTPAdapter(pool_connections=20,
-                                                pool_maxsize=20,
-                                                max_retries=1)
-        session = requests.Session()
-        if proxy:
-            session.proxies = proxy
-        elif use_proxy:
-            if is_static_proxy == True:
-                session.proxies = get_static_http_proxy()
-            else:
-                session.proxies = get_dynamic_http_proxy()
-        session.mount('http://', adapter)
-        session.mount('https://', adapter)
         if not endpoint:
             raise ValueError("rpc cant be empty")
-        w3_provider = Web3.HTTPProvider(endpoint_uri=endpoint, session=session)
-        return Web3(provider=w3_provider)
+        if not proxy and use_proxy:
+            proxy = get_static_http_proxy(
+            ) if is_static_proxy else get_dynamic_http_proxy()
+        provider = AsyncHTTPWithProxyProvider(
+            endpoint_uri=endpoint,
+            proxy=proxy) if proxy else AsyncHTTPProvider(endpoint_uri=endpoint)
+        return AsyncWeb3(provider=provider)
 
     def init_account(self, private_key: str) -> Account:
         return Account.from_key(private_key)
@@ -144,7 +139,7 @@ class EvmAccount:
         return to_checksum_address(addr)
 
     def init_contract(self, addr: Address, dir: str,
-                      abi_name: str) -> Contract:
+                      abi_name: str) -> AsyncContract:
         with Path(dir).joinpath(abi_name).open('r') as f:
             abi = f.read()
         return self.init_contract_with_abi_str(addr=addr, abi_str=abi)
@@ -152,51 +147,50 @@ class EvmAccount:
     def init_contract_with_abi_str(self, addr: Address, abi_str: str):
         return self.w3.eth.contract(address=addr, abi=abi_str)
 
-    def get_balance(self, address: Address = None):
+    async def get_balance(self, address: Address = None):
         if address is None:
             address = self.account.address
-        return self.w3.eth.get_balance(address)
+        return await self.w3.eth.get_balance(address)
 
-    def get_token_balance(self,
-                          token_address: Address,
-                          address: Address = None):
+    async def get_token_balance(self,
+                                token_address: Address,
+                                address: Address = None):
         if address is None:
             address = self.account.address
         erc20_token = self.init_contract(addr=token_address,
                                          dir=os.path.dirname(__file__),
                                          abi_name='erc20.json')
-        return erc20_token.functions.balanceOf(address).call()
+        return await erc20_token.functions.balanceOf(address).call()
 
-    def transfer_token(self, token_address: Address,
-                       receipient_address: Address, value: Wei):
+    async def transfer_token(self, token_address: Address,
+                             receipient_address: Address, value: Wei):
         erc20_token = self.init_contract(addr=token_address,
                                          dir=os.path.dirname(__file__),
                                          abi_name='erc20.json')
         f = erc20_token.functions.transfer(receipient_address, value)
-        return self.make_tx(f=f)
+        return await self.make_tx(f=f)
 
-    def get_nonce(self, address: str = None):
+    async def get_nonce(self, address: str = None):
         addr = self.address
         if address is not None:
             addr = address
-        return self.w3.eth.get_transaction_count(addr, 'pending')
+        return await self.w3.eth.get_transaction_count(addr, 'pending')
 
-    def get_gas_price(self) -> Wei:
-        gas_price = self.w3.eth.gas_price
-        return gas_price
+    async def get_gas_price(self) -> Wei:
+        return await self.w3.eth.gas_price
 
-    def get_default_tx_params(self,
-                              chain_id: str = None,
-                              to: Address = None,
-                              data: str = None,
-                              val: Wei = Wei(0),
-                              gas: Optional[Wei] = None,
-                              gas_price: Optional[Wei] = None):
+    async def get_default_tx_params(self,
+                                    chain_id: str = None,
+                                    to: Address = None,
+                                    data: str = None,
+                                    val: Wei = Wei(0),
+                                    gas: Optional[Wei] = None,
+                                    gas_price: Optional[Wei] = None):
 
         tx_params: TxParams = {
             'from': self.account.address,
             'value': val,
-            'nonce': self.get_nonce()
+            'nonce': await self.get_nonce()
         }
         if gas is not None:
             tx_params['gas'] = gas
@@ -210,19 +204,21 @@ class EvmAccount:
             tx_params['chainId'] = chain_id
         return tx_params
 
-    def sign_tx(self, params: TxParams):
-        return self.w3.eth.account.sign_transaction(
-            params, private_key=self.private_key)
+    async def sign_tx(self, params: TxParams):
+        signed_tx = await asyncio.to_thread(
+            self.w3.eth.account.sign_transaction, params, self.private_key)
+        return signed_tx
 
-    def send_tx_and_wait_recipt(self,
-                                signed_tx,
-                                timeout: int = 120,
-                                wait_for_tx: bool = True) -> Tuple:
+    async def send_tx_and_wait_recipt(self,
+                                      signed_tx,
+                                      timeout: int = 120,
+                                      wait_for_tx: bool = True) -> Tuple:
         try:
-            tx = self.w3.eth.send_raw_transaction(signed_tx.raw_transaction)
+            tx = await self.w3.eth.send_raw_transaction(
+                signed_tx.raw_transaction)
             txHash = to_hex(tx)
             if wait_for_tx:
-                recipt = self.w3.eth.wait_for_transaction_receipt(
+                recipt = await self.w3.eth.wait_for_transaction_receipt(
                     tx, timeout=timeout)
                 if recipt == None:
                     self.log_err(f'tx failed: {txHash}')
@@ -240,31 +236,31 @@ class EvmAccount:
             self.log_err(f'tx exception occurs : {e}')
             return None, False
 
-    def simulate_tx(self,
-                    chain_id: str = None,
-                    f: ContractFunction = None,
-                    gas: Optional[Wei] = None,
-                    gas_price: Optional[Wei] = None,
-                    to: Address = None,
-                    data: str = None,
-                    val: Wei = Wei(0)):
+    async def simulate_tx(self,
+                          chain_id: str = None,
+                          f: AsyncContractFunction = None,
+                          gas: Optional[Wei] = None,
+                          gas_price: Optional[Wei] = None,
+                          to: Address = None,
+                          data: str = None,
+                          val: Wei = Wei(0)):
 
-        tx_params = self.get_default_tx_params(chain_id=chain_id,
-                                               to=to,
-                                               data=data,
-                                               val=val,
-                                               gas=gas,
-                                               gas_price=gas_price)
+        tx_params = await self.get_default_tx_params(chain_id=chain_id,
+                                                     to=to,
+                                                     data=data,
+                                                     val=val,
+                                                     gas=gas,
+                                                     gas_price=gas_price)
         if gas_price is None:
-            gas_price = self.w3.eth.gas_price
+            gas_price = await self.w3.eth.gas_price
             self.log_debug(f'estimated gas price: {gas_price/1e9} GWei')
             tx_params['gasPrice'] = int(self.gas_price_multiplier * gas_price)
 
         if f is not None:
-            tx_params = f.build_transaction(transaction=tx_params)
+            tx_params = await f.build_transaction(transaction=tx_params)
 
         if gas is None:
-            gas = self.w3.eth.estimate_gas(tx_params)
+            gas = await self.w3.eth.estimate_gas(tx_params)
             self.log_debug(f'estimated gas limit: {gas}')
             tx_params['gas'] = int(gas * self.gas_limit_multiplier)
         self.log_debug(f'[origin] estimated gas fee: {gas_price*gas/1e18}')
@@ -274,49 +270,80 @@ class EvmAccount:
             )
         return tx_params
 
-    def make_tx(self,
-                chain_id: str = None,
-                f: ContractFunction = None,
-                gas: Optional[Wei] = None,
-                gas_price: Optional[Wei] = None,
-                to: Address = None,
-                data: str = None,
-                val: Wei = Wei(0),
-                timeout: int = 120,
-                wait_for_tx: bool = True):
+    async def make_tx(self,
+                      chain_id: str = None,
+                      f: AsyncContractFunction = None,
+                      gas: Optional[Wei] = None,
+                      gas_price: Optional[Wei] = None,
+                      to: Address = None,
+                      data: str = None,
+                      val: Wei = Wei(0),
+                      timeout: int = 120,
+                      wait_for_tx: bool = True):
         try:
-            tx_params = self.simulate_tx(chain_id=chain_id,
-                                         f=f,
-                                         gas=gas,
-                                         gas_price=gas_price,
-                                         to=to,
-                                         data=data,
-                                         val=val)
+            tx_params = await self.simulate_tx(chain_id=chain_id,
+                                               f=f,
+                                               gas=gas,
+                                               gas_price=gas_price,
+                                               to=to,
+                                               data=data,
+                                               val=val)
         except Exception as e:
             self.log_err(f'tx simulated failed, err = {e}')
             return None, False
-        signed_tx = self.sign_tx(params=tx_params)
-        return self.send_tx_and_wait_recipt(signed_tx=signed_tx,
-                                            timeout=timeout,
-                                            wait_for_tx=wait_for_tx)
+        signed_tx = await self.sign_tx(params=tx_params)
+        return await self.send_tx_and_wait_recipt(signed_tx=signed_tx,
+                                                  timeout=timeout,
+                                                  wait_for_tx=wait_for_tx)
 
 
 if __name__ == '__main__':
 
-    sepolia_test_pk = '0xxxxxxxx'
-    from .const import SEPOLIA
-    wallet = EvmAccount(private_key=sepolia_test_pk, endpoint_name=SEPOLIA)
+    async def test():
+        from const import SEPOLIA
 
-    nonce = wallet.get_nonce()
-    wallet.log_info(f'nonce = {nonce}')
+        def create_accounts(n: int = 100):
+            from eth_account import Account
+            accounts = []
+            for i in range(n):
+                accounts.append(Account.create().key.hex())
+            return accounts
 
-    gas_price = wallet.get_gas_price()
-    wallet.log_info(f'gas_price = {round(gas_price/1e9,2)} GWei')
-    from .util import to_checksum_address
-    wallet.simulate_tx(
-        to=to_checksum_address('0x5c9b7a5c7c8e5f4c5a8e5f4c5a8e5f4c5a8e5f4d'),
-        val=Web3.to_wei(0.0001, 'ether'))
+        async def get_nonce(wallet: EvmAccount):
+            try:
+                nonce = await wallet.get_nonce()
+                wallet.log_info(f'nonce = {nonce}')
+            except Exception as e:
+                wallet.log_err(f'nonce exception occurs : {e}')
 
-    wallet.make_tx(
-        to=to_checksum_address('0x5c9b7a5c7c8e5f4c5a8e5f4c5a8e5f4c5a8e5f4d'),
-        val=Web3.to_wei(0.0001, 'ether'))
+        async def gas_price(wallet: EvmAccount):
+            try:
+                nonce = await wallet.get_gas_price()
+                wallet.log_info(f'gas_price = {nonce}')
+            except Exception as e:
+                wallet.log_err(f'gas_price exception occurs : {e}')
+
+        async def simulate(wallet: EvmAccount):
+            try:
+                from util import to_checksum_address
+                return await wallet.simulate_tx(to=to_checksum_address(
+                    '0x5c9b7a5c7c8e5f4c5a8e5f4c5a8e5f4c5a8e5f4d'),
+                                                val=Web3.to_wei(
+                                                    0.0001, 'ether'))
+            except Exception as e:
+                wallet.log_err(f'simulate exception occurs : {e}')
+
+        tasks = []
+
+        for i, pk in enumerate(create_accounts(100)):
+            wallet = EvmAccount(name=f'{i+1}',
+                                private_key=pk,
+                                endpoint_name=SEPOLIA)
+            tasks.extend(
+                [get_nonce(wallet),
+                 gas_price(wallet),
+                 simulate(wallet)])
+
+        await asyncio.gather(*tasks)
+
+    asyncio.run(test())
